@@ -1,4 +1,4 @@
-# Copyright 2014 Google Inc.
+# Copyright 2014 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,23 +16,22 @@
 
 import base64
 
+from google.api_core import page_iterator
 from google.cloud._helpers import _ensure_tuple_or_list
-from google.cloud.iterator import Iterator as BaseIterator
-from google.cloud.iterator import Page
 
-from google.cloud.proto.datastore.v1 import datastore_pb2 as _datastore_pb2
-from google.cloud.proto.datastore.v1 import entity_pb2 as _entity_pb2
-from google.cloud.proto.datastore.v1 import query_pb2 as _query_pb2
+from google.cloud.datastore_v1.proto import entity_pb2
+from google.cloud.datastore_v1.proto import query_pb2
 from google.cloud.datastore import helpers
 from google.cloud.datastore.key import Key
 
 
-_NOT_FINISHED = _query_pb2.QueryResultBatch.NOT_FINISHED
+_NOT_FINISHED = query_pb2.QueryResultBatch.NOT_FINISHED
+_NO_MORE_RESULTS = query_pb2.QueryResultBatch.NO_MORE_RESULTS
 
 _FINISHED = (
-    _query_pb2.QueryResultBatch.NO_MORE_RESULTS,
-    _query_pb2.QueryResultBatch.MORE_RESULTS_AFTER_LIMIT,
-    _query_pb2.QueryResultBatch.MORE_RESULTS_AFTER_CURSOR,
+    _NO_MORE_RESULTS,
+    query_pb2.QueryResultBatch.MORE_RESULTS_AFTER_LIMIT,
+    query_pb2.QueryResultBatch.MORE_RESULTS_AFTER_CURSOR,
 )
 
 
@@ -82,24 +81,26 @@ class Query(object):
     """
 
     OPERATORS = {
-        '<=': _query_pb2.PropertyFilter.LESS_THAN_OR_EQUAL,
-        '>=': _query_pb2.PropertyFilter.GREATER_THAN_OR_EQUAL,
-        '<': _query_pb2.PropertyFilter.LESS_THAN,
-        '>': _query_pb2.PropertyFilter.GREATER_THAN,
-        '=': _query_pb2.PropertyFilter.EQUAL,
+        "<=": query_pb2.PropertyFilter.LESS_THAN_OR_EQUAL,
+        ">=": query_pb2.PropertyFilter.GREATER_THAN_OR_EQUAL,
+        "<": query_pb2.PropertyFilter.LESS_THAN,
+        ">": query_pb2.PropertyFilter.GREATER_THAN,
+        "=": query_pb2.PropertyFilter.EQUAL,
     }
     """Mapping of operator strings and their protobuf equivalents."""
 
-    def __init__(self,
-                 client,
-                 kind=None,
-                 project=None,
-                 namespace=None,
-                 ancestor=None,
-                 filters=(),
-                 projection=(),
-                 order=(),
-                 distinct_on=()):
+    def __init__(
+        self,
+        client,
+        kind=None,
+        project=None,
+        namespace=None,
+        ancestor=None,
+        filters=(),
+        projection=(),
+        order=(),
+        distinct_on=(),
+    ):
 
         self._client = client
         self._kind = kind
@@ -110,9 +111,9 @@ class Query(object):
         # Verify filters passed in.
         for property_name, operator, value in filters:
             self.add_filter(property_name, operator, value)
-        self._projection = _ensure_tuple_or_list('projection', projection)
-        self._order = _ensure_tuple_or_list('order', order)
-        self._distinct_on = _ensure_tuple_or_list('distinct_on', distinct_on)
+        self._projection = _ensure_tuple_or_list("projection", projection)
+        self._order = _ensure_tuple_or_list("order", order)
+        self._distinct_on = _ensure_tuple_or_list("distinct_on", distinct_on)
 
     @property
     def project(self):
@@ -219,8 +220,8 @@ class Query(object):
           >>> from google.cloud import datastore
           >>> client = datastore.Client()
           >>> query = client.query(kind='Person')
-          >>> query.add_filter('name', '=', 'James')
-          >>> query.add_filter('age', '>', 50)
+          >>> query = query.add_filter('name', '=', 'James')
+          >>> query = query.add_filter('age', '>', 50)
 
         :type property_name: str
         :param property_name: A property name.
@@ -234,19 +235,23 @@ class Query(object):
                      :class:`google.cloud.datastore.key.Key`
         :param value: The value to filter on.
 
+        :rtype: :class:`~google.cloud.datastore.query.Query`
+        :returns: A query object.
+
         :raises: :class:`ValueError` if ``operation`` is not one of the
                  specified values, or if a filter names ``'__key__'`` but
                  passes an invalid value (a key is required).
         """
         if self.OPERATORS.get(operator) is None:
             error_message = 'Invalid expression: "%s"' % (operator,)
-            choices_message = 'Please use one of: =, <, <=, >, >=.'
+            choices_message = "Please use one of: =, <, <=, >, >=."
             raise ValueError(error_message, choices_message)
 
-        if property_name == '__key__' and not isinstance(value, Key):
+        if property_name == "__key__" and not isinstance(value, Key):
             raise ValueError('Invalid key: "%s"' % value)
 
         self._filters.append((property_name, operator, value))
+        return self
 
     @property
     def projection(self):
@@ -271,9 +276,9 @@ class Query(object):
 
     def keys_only(self):
         """Set the projection to include only keys."""
-        self._projection[:] = ['__key__']
+        self._projection[:] = ["__key__"]
 
-    def key_filter(self, key, operator='='):
+    def key_filter(self, key, operator="="):
         """Filter on a key.
 
         :type key: :class:`google.cloud.datastore.key.Key`
@@ -283,7 +288,7 @@ class Query(object):
         :param operator: (Optional) One of ``=``, ``<``, ``<=``, ``>``, ``>=``.
                          Defaults to ``=``.
         """
-        self.add_filter('__key__', operator, key)
+        self.add_filter("__key__", operator, key)
 
     @property
     def order(self):
@@ -331,8 +336,15 @@ class Query(object):
             value = [value]
         self._distinct_on[:] = value
 
-    def fetch(self, limit=None, offset=0, start_cursor=None, end_cursor=None,
-              client=None):
+    def fetch(
+        self,
+        limit=None,
+        offset=0,
+        start_cursor=None,
+        end_cursor=None,
+        client=None,
+        eventual=False,
+    ):
         """Execute the Query; return an iterator for the matching entities.
 
         For example::
@@ -340,8 +352,8 @@ class Query(object):
           >>> from google.cloud import datastore
           >>> client = datastore.Client()
           >>> query = client.query(kind='Person')
-          >>> query.add_filter('name', '=', 'Sally')
-          >>> list(query.fetch())
+          >>> result = query.add_filter('name', '=', 'Sally').fetch()
+          >>> list(result)
           [<Entity object>, <Entity object>, ...]
           >>> list(query.fetch(1))
           [<Entity object>]
@@ -359,9 +371,15 @@ class Query(object):
         :param end_cursor: (Optional) cursor passed through to the iterator.
 
         :type client: :class:`google.cloud.datastore.client.Client`
-        :param client: client used to connect to datastore.
+        :param client: (Optional) client used to connect to datastore.
                        If not supplied, uses the query's value.
 
+        :type eventual: bool
+        :param eventual: (Optional) Defaults to strongly consistent (False).
+                                    Setting True will use eventual consistency,
+                                    but cannot be used inside a transaction or
+                                    will raise ValueError.
+
         :rtype: :class:`Iterator`
         :returns: The iterator for the query.
         """
@@ -369,11 +387,17 @@ class Query(object):
             client = self._client
 
         return Iterator(
-            self, client, limit=limit, offset=offset,
-            start_cursor=start_cursor, end_cursor=end_cursor)
+            self,
+            client,
+            limit=limit,
+            offset=offset,
+            start_cursor=start_cursor,
+            end_cursor=end_cursor,
+            eventual=eventual,
+        )
 
 
-class Iterator(BaseIterator):
+class Iterator(page_iterator.Iterator):
     """Represent the state of a given execution of a Query.
 
     :type query: :class:`~google.cloud.datastore.query.Query`
@@ -397,18 +421,36 @@ class Iterator(BaseIterator):
     :type end_cursor: bytes
     :param end_cursor: (Optional) Cursor to end paging through
                        query results.
+
+    :type eventual: bool
+    :param eventual: (Optional) Defaults to strongly consistent (False).
+                                Setting True will use eventual consistency,
+                                but cannot be used inside a transaction or
+                                will raise ValueError.
     """
 
     next_page_token = None
 
-    def __init__(self, query, client, limit=None, offset=None,
-                 start_cursor=None, end_cursor=None):
+    def __init__(
+        self,
+        query,
+        client,
+        limit=None,
+        offset=None,
+        start_cursor=None,
+        end_cursor=None,
+        eventual=False,
+    ):
         super(Iterator, self).__init__(
-            client=client, item_to_value=_item_to_entity,
-            page_token=start_cursor, max_results=limit)
+            client=client,
+            item_to_value=_item_to_entity,
+            page_token=start_cursor,
+            max_results=limit,
+        )
         self._query = query
         self._offset = offset
         self._end_cursor = end_cursor
+        self._eventual = eventual
         # The attributes below will change over the life of the iterator.
         self._more_results = True
         self._skipped_results = 0
@@ -436,10 +478,12 @@ class Iterator(BaseIterator):
         if self.max_results is not None:
             pb.limit.value = self.max_results - self.num_results
 
-        if self._offset is not None:
-            # NOTE: The offset goes down relative to the location
-            #       because we are updating the cursor each time.
-            pb.offset = self._offset - self._skipped_results
+        if start_cursor is None and self._offset is not None:
+            # NOTE: We don't need to add an offset to the request protobuf
+            #       if we are using an existing cursor, because the offset
+            #       is only relative to the start of the result set, not
+            #       relative to each page (this method is called per-page)
+            pb.offset = self._offset
 
         return pb
 
@@ -454,12 +498,12 @@ class Iterator(BaseIterator):
         :raises ValueError: If ``more_results`` is an unexpected value.
         """
         self._skipped_results = response_pb.batch.skipped_results
-
-        if response_pb.batch.end_cursor == b'':  # Empty-value for bytes.
+        if response_pb.batch.more_results == _NO_MORE_RESULTS:
             self.next_page_token = None
         else:
             self.next_page_token = base64.urlsafe_b64encode(
-                response_pb.batch.end_cursor)
+                response_pb.batch.end_cursor
+            )
         self._end_cursor = None
 
         if response_pb.batch.more_results == _NOT_FINISHED:
@@ -467,7 +511,7 @@ class Iterator(BaseIterator):
         elif response_pb.batch.more_results in _FINISHED:
             self._more_results = False
         else:
-            raise ValueError('Unexpected value returned for `more_results`.')
+            raise ValueError("Unexpected value returned for `more_results`.")
 
         return [result.entity for result in response_pb.batch.entity_results]
 
@@ -484,22 +528,34 @@ class Iterator(BaseIterator):
         query_pb = self._build_protobuf()
         transaction = self.client.current_transaction
         if transaction is None:
-            read_options = _datastore_pb2.ReadOptions()
+            transaction_id = None
         else:
-            read_options = _datastore_pb2.ReadOptions(
-                transaction=transaction.id)
+            transaction_id = transaction.id
+        read_options = helpers.get_read_options(self._eventual, transaction_id)
 
-        partition_id = _entity_pb2.PartitionId(
-            project_id=self._query.project,
-            namespace_id=self._query.namespace)
-        response_pb = self.client._datastore_api.run_query(
-            self._query.project,
-            partition_id,
-            read_options,
-            query=query_pb,
+        partition_id = entity_pb2.PartitionId(
+            project_id=self._query.project, namespace_id=self._query.namespace
         )
+        response_pb = self.client._datastore_api.run_query(
+            self._query.project, partition_id, read_options, query=query_pb
+        )
+
+        while (
+            response_pb.batch.more_results == _NOT_FINISHED
+            and response_pb.batch.skipped_results < query_pb.offset
+        ):
+            # We haven't finished processing. A likely reason is we haven't
+            # skipped all of the results yet. Don't return any results.
+            # Instead, rerun query, adjusting offsets. Datastore doesn't process
+            # more than 1000 skipped results in a query.
+            query_pb.start_cursor = response_pb.batch.skipped_cursor
+            query_pb.offset -= response_pb.batch.skipped_results
+            response_pb = self.client._datastore_api.run_query(
+                self._query.project, partition_id, read_options, query=query_pb
+            )
+
         entity_pbs = self._process_query_results(response_pb)
-        return Page(self, entity_pbs, self._item_to_value)
+        return page_iterator.Page(self, entity_pbs, self.item_to_value)
 
 
 def _pb_from_query(query):
@@ -513,7 +569,7 @@ def _pb_from_query(query):
               it does not contain "in-flight" fields for ongoing query
               executions (cursors, offset, limit).
     """
-    pb = _query_pb2.Query()
+    pb = query_pb2.Query()
 
     for projection_name in query.projection:
         pb.projection.add().property.name = projection_name
@@ -522,15 +578,15 @@ def _pb_from_query(query):
         pb.kind.add().name = query.kind
 
     composite_filter = pb.filter.composite_filter
-    composite_filter.op = _query_pb2.CompositeFilter.AND
+    composite_filter.op = query_pb2.CompositeFilter.AND
 
     if query.ancestor:
         ancestor_pb = query.ancestor.to_protobuf()
 
         # Filter on __key__ HAS_ANCESTOR == ancestor.
         ancestor_filter = composite_filter.filters.add().property_filter
-        ancestor_filter.property.name = '__key__'
-        ancestor_filter.op = _query_pb2.PropertyFilter.HAS_ANCESTOR
+        ancestor_filter.property.name = "__key__"
+        ancestor_filter.op = query_pb2.PropertyFilter.HAS_ANCESTOR
         ancestor_filter.value.key_value.CopyFrom(ancestor_pb)
 
     for property_name, operator, value in query.filters:
@@ -542,19 +598,19 @@ def _pb_from_query(query):
         property_filter.op = pb_op_enum
 
         # Set the value to filter on based on the type.
-        if property_name == '__key__':
+        if property_name == "__key__":
             key_pb = value.to_protobuf()
             property_filter.value.key_value.CopyFrom(key_pb)
         else:
             helpers._set_protobuf_value(property_filter.value, value)
 
     if not composite_filter.filters:
-        pb.ClearField('filter')
+        pb.ClearField("filter")
 
     for prop in query.order:
         property_order = pb.order.add()
 
-        if prop.startswith('-'):
+        if prop.startswith("-"):
             property_order.property.name = prop[1:]
             property_order.direction = property_order.DESCENDING
         else:
@@ -571,7 +627,7 @@ def _pb_from_query(query):
 def _item_to_entity(iterator, entity_pb):
     """Convert a raw protobuf entity to the native object.
 
-    :type iterator: :class:`~google.cloud.iterator.Iterator`
+    :type iterator: :class:`~google.api_core.page_iterator.Iterator`
     :param iterator: The iterator that is currently in use.
 
     :type entity_pb:
@@ -582,4 +638,6 @@ def _item_to_entity(iterator, entity_pb):
     :returns: The next entity in the page.
     """
     return helpers.entity_from_protobuf(entity_pb)
+
+
 # pylint: enable=unused-argument
